/* THIS FILE IS AUTOGENERATED FROM StorageType.webidl BY Codegen.py - DO NOT EDIT */

#ifndef mozilla_dom_StorageTypeBinding_h
#define mozilla_dom_StorageTypeBinding_h

#include "js/RootingAPI.h"
#include "jspubtd.h"
#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/Nullable.h"

namespace mozilla {
namespace dom {

struct NativePropertyHooks;
class ProtoAndIfaceCache;

} // namespace dom
} // namespace mozilla

namespace mozilla {
namespace dom {

enum class StorageType : uint32_t {
  Persistent,
  Temporary,
  Default,
  EndGuard_
};

namespace StorageTypeValues {
extern const EnumEntry strings[4];
} // namespace StorageTypeValues

bool
ToJSValue(JSContext* aCx, StorageType aArgument, JS::MutableHandle<JS::Value> aValue);


} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_StorageTypeBinding_h
